@@ -2,36 +2,40 @@
 Export some objects that are used by Textual and that help document other features.
 """
 
-from ._animator import Animatable, EasingFunction
-from ._context import NoActiveAppError
-from ._path import CSSPathError, CSSPathType
-from ._types import (
+from textual._animator import Animatable, EasingFunction
+from textual._context import NoActiveAppError
+from textual._path import CSSPathError, CSSPathType
+from textual._types import (
+    AnimationLevel,
     CallbackType,
     IgnoreReturnCallbackType,
     MessageTarget,
     UnusedParameter,
     WatchCallbackType,
 )
-from .actions import ActionParseResult
-from .css.styles import RenderStyles
-from .widgets._directory_tree import DirEntry
-from .widgets._input import InputValidationOn
-from .widgets._option_list import (
+from textual._widget_navigation import Direction
+from textual.actions import ActionParseResult
+from textual.css.styles import RenderStyles
+from textual.widgets._directory_tree import DirEntry
+from textual.widgets._input import InputValidationOn
+from textual.widgets._option_list import (
     DuplicateID,
     NewOptionListContent,
     OptionDoesNotExist,
     OptionListContent,
 )
-from .widgets._placeholder import PlaceholderVariant
-from .widgets._select import NoSelection, SelectType
+from textual.widgets._placeholder import PlaceholderVariant
+from textual.widgets._select import NoSelection, SelectType
 
 __all__ = [
     "ActionParseResult",
     "Animatable",
+    "AnimationLevel",
     "CallbackType",
     "CSSPathError",
     "CSSPathType",
     "DirEntry",
+    "Direction",
     "DuplicateID",
     "EasingFunction",
     "IgnoreReturnCallbackType",
