@@ -1,6 +1,5 @@
 from __future__ import annotations
 
-import asyncio
 from dataclasses import dataclass
 from typing import ClassVar
 
@@ -9,19 +8,19 @@ from rich.console import RenderableType
 from rich.style import Style
 from rich.text import Text, TextType
 
-from .. import events
-from ..app import ComposeResult, RenderResult
-from ..await_complete import AwaitComplete
-from ..binding import Binding, BindingType
-from ..containers import Container, Horizontal, Vertical
-from ..css.query import NoMatches
-from ..events import Mount
-from ..geometry import Offset
-from ..message import Message
-from ..reactive import reactive
-from ..renderables.bar import Bar
-from ..widget import Widget
-from ..widgets import Static
+from textual import events
+from textual.app import ComposeResult, RenderResult
+from textual.await_complete import AwaitComplete
+from textual.binding import Binding, BindingType
+from textual.containers import Container, Horizontal, Vertical
+from textual.css.query import NoMatches
+from textual.events import Mount
+from textual.geometry import Offset
+from textual.message import Message
+from textual.reactive import reactive
+from textual.renderables.bar import Bar
+from textual.widget import Widget
+from textual.widgets import Static
 
 
 class Underline(Widget):
@@ -31,10 +30,13 @@ class Underline(Widget):
     Underline {
         width: 1fr;
         height: 1;
-    }
-    Underline > .underline--bar {
-        background: $foreground 10%;
-        color: $accent;
+        & > .underline--bar {
+            color: $block-cursor-background;
+            background: $foreground 10%;
+        }
+        &:ansi {
+            text-style: dim;
+        }
     }
     """
 
@@ -42,7 +44,7 @@ class Underline(Widget):
     """
     | Class | Description |
     | :- | :- |
-    | `underline-bar` | Style of the bar (may be used to change the color). |
+    | `underline--bar` | Style of the bar (may be used to change the color). |
     """
 
     highlight_start = reactive(0)
@@ -92,27 +94,24 @@ class Tab(Static):
     DEFAULT_CSS = """
     Tab {
         width: auto;
-        height: 2;
-        padding: 1 1 0 2;
+        height: 1;
+        padding: 0 1;
         text-align: center;
-        color: $text-disabled;
-    }
-    Tab.-active {
-        text-style: bold;
-        color: $text;
-    }
-    Tab:hover {
-        text-style: bold;
-    }
-    Tab.-active:hover {
-        color: $text;
-    }
-    Tab:disabled {
-        color: $text-disabled;
-        text-opacity: 50%;
-    }
-    Tab.-hidden {
-        display: none;
+        color: $foreground 50%;
+
+        &:hover {
+            color: $foreground;
+        }
+        &:disabled {
+            color: $foreground 25%;
+        }
+
+        &.-active {
+            color: $foreground;
+        }
+        &.-hidden {
+            display: none;
+        }
     }
     """
 
@@ -165,7 +164,8 @@ class Tab(Static):
         """
         super().__init__(id=id, classes=classes, disabled=disabled)
         self._label: Text
-        self.label = label
+        # Setter takes Text or str
+        self.label = label  # type: ignore[assignment]
 
     @property
     def label(self) -> Text:
@@ -201,23 +201,52 @@ class Tabs(Widget, can_focus=True):
     DEFAULT_CSS = """
     Tabs {
         width: 100%;
-        height: 3;
-    }
-    Tabs > #tabs-scroll {
-        overflow: hidden;
-    }
-    Tabs #tabs-list {
-       width: auto;
-       min-height: 2;
-    }
-    Tabs #tabs-list-bar, Tabs #tabs-list {
-        width: auto;
-        height: auto;
-        min-width: 100%;
-        overflow: hidden hidden;
-    }
-    Tabs:focus .underline--bar {
-        background: $foreground 20%;
+        height: 2;
+        &:focus {
+            .underline--bar {
+                background: $foreground 30%;
+            }
+            & .-active {
+                text-style: $block-cursor-text-style;
+                color: $block-cursor-foreground;
+                background: $block-cursor-background;
+            }
+        }
+
+        & > #tabs-scroll {
+            overflow: hidden;
+        }
+
+        #tabs-list {
+            width: auto;
+        }
+        #tabs-list-bar, #tabs-list {
+            width: auto;
+            height: auto;
+            min-width: 100%;
+            overflow: hidden hidden;
+        }
+        &:ansi {
+            #tabs-list {
+                text-style: dim;
+            }
+            & #tabs-list > .-active {
+                text-style: not dim;
+            }
+            &:focus {
+                #tabs-list > .-active {
+                    text-style: bold not dim;
+                }
+            }
+            & .underline--bar {
+                color: ansi_bright_blue;
+                background: ansi_default;
+            }
+            & .-active {
+                color: transparent;
+                background: transparent;
+            }
+        }
     }
     """
 
@@ -285,7 +314,8 @@ class Tabs(Widget, can_focus=True):
     class Cleared(Message):
         """Sent when there are no active tabs.
 
-        This can occur when Tabs are cleared, or if all tabs are hidden.
+        This can occur when Tabs are cleared, if all tabs are hidden, or if the
+        currently active tab is unset.
         """
 
         def __init__(self, tabs: Tabs) -> None:
@@ -327,10 +357,10 @@ class Tabs(Widget, can_focus=True):
         Args:
             *tabs: Positional argument should be explicit Tab objects, or a str or Text.
             active: ID of the tab which should be active on start.
-            name: Optional name for the input widget.
+            name: Optional name for the tabs widget.
             id: Optional ID for the widget.
             classes: Optional initial classes for the widget.
-            disabled: Whether the input is disabled or not.
+            disabled: Whether the widget is disabled or not.
         """
         self._tabs_counter = 0
 
@@ -496,7 +526,7 @@ class Tabs(Widget, can_focus=True):
         underline.highlight_end = 0
         self.call_after_refresh(self.post_message, self.Cleared(self))
         self.active = ""
-        return AwaitComplete(self.query("#tabs-list > Tab").remove()())
+        return AwaitComplete(self.query("#tabs-list > Tab").remove())
 
     def remove_tab(self, tab_or_id: Tab | str | None) -> AwaitComplete:
         """Remove a tab.
@@ -508,7 +538,7 @@ class Tabs(Widget, can_focus=True):
             An optionally awaitable object that waits for the tab to be removed.
         """
         if not tab_or_id:
-            return AwaitComplete(self.app._remove_nodes([], None)())
+            return AwaitComplete()
 
         if isinstance(tab_or_id, Tab):
             remove_tab = tab_or_id
@@ -516,29 +546,24 @@ class Tabs(Widget, can_focus=True):
             try:
                 remove_tab = self.query_one(f"#tabs-list > #{tab_or_id}", Tab)
             except NoMatches:
-                return AwaitComplete(self.app._remove_nodes([], None)())
+                return AwaitComplete()
 
-        removing_active_tab = remove_tab.has_class("-active")
-        next_tab = self._next_active
-        remove_await = remove_tab.remove()
-
-        highlight_updated = asyncio.Event()
+        if remove_tab.has_class("-active"):
+            next_tab = self._next_active
+        else:
+            next_tab = None
 
         async def do_remove() -> None:
             """Perform the remove after refresh so the underline bar gets new positions."""
-            await remove_await
-            if next_tab is None:
+            await remove_tab.remove()
+            if not self.query("#tabs-list > Tab"):
                 self.active = ""
-            elif removing_active_tab:
-                self.active = next_tab.id
-                next_tab.add_class("-active")
+            elif next_tab is not None:
+                self.active = next_tab.id or ""
+            else:
+                self._highlight_active(animate=False)
 
-            highlight_updated.set()
-
-        async def wait_for_highlight_update() -> None:
-            await highlight_updated.wait()
-
-        return AwaitComplete(do_remove(), wait_for_highlight_update())
+        return AwaitComplete(do_remove())
 
     def validate_active(self, active: str) -> str:
         """Check id assigned to active attribute is a valid tab."""
@@ -575,14 +600,16 @@ class Tabs(Widget, can_focus=True):
 
     def watch_active(self, previously_active: str, active: str) -> None:
         """Handle a change to the active tab."""
+        self.query("#tabs-list > Tab.-active").remove_class("-active")
         if active:
             try:
                 active_tab = self.query_one(f"#tabs-list > #{active}", Tab)
             except NoMatches:
                 return
-            self.query("#tabs-list > Tab.-active").remove_class("-active")
             active_tab.add_class("-active")
+
             self._highlight_active(animate=previously_active != "")
+
             self._scroll_active_tab()
             self.post_message(self.TabActivated(self, active_tab))
         else:
@@ -602,29 +629,30 @@ class Tabs(Widget, can_focus=True):
         """
         underline = self.query_one(Underline)
         try:
-            active_tab = self.query_one(f"#tabs-list > Tab.-active")
+            _active_tab = self.query_one("#tabs-list > Tab.-active")
         except NoMatches:
             underline.show_highlight = False
             underline.highlight_start = 0
             underline.highlight_end = 0
         else:
             underline.show_highlight = True
-            tab_region = active_tab.virtual_region.shrink(active_tab.styles.gutter)
-            start, end = tab_region.column_span
-            # This is a basic animation, so we only disable it if we want no animations.
-            if animate and self.app.animation_level != "none":
 
-                def animate_underline() -> None:
-                    """Animate the underline."""
-                    try:
-                        active_tab = self.query_one(f"#tabs-list > Tab.-active")
-                    except NoMatches:
-                        pass
-                    else:
-                        tab_region = active_tab.virtual_region.shrink(
-                            active_tab.styles.gutter
-                        )
-                        start, end = tab_region.column_span
+            def move_underline(animate: bool) -> None:
+                """Move the tab underline.
+
+                Args:
+                    animate: animate the underline to its new position.
+                """
+                try:
+                    active_tab = self.query_one("#tabs-list > Tab.-active")
+                except NoMatches:
+                    pass
+                else:
+                    tab_region = active_tab.virtual_region.shrink(
+                        active_tab.styles.gutter
+                    )
+                    start, end = tab_region.column_span
+                    if animate:
                         underline.animate(
                             "highlight_start",
                             start,
@@ -637,11 +665,17 @@ class Tabs(Widget, can_focus=True):
                             duration=0.3,
                             level="basic",
                         )
+                    else:
+                        underline.highlight_start = start
+                        underline.highlight_end = end
 
-                self.set_timer(0.02, lambda: self.call_after_refresh(animate_underline))
+            if animate and self.app.animation_level != "none":
+                self.set_timer(
+                    0.02,
+                    lambda: self.call_after_refresh(move_underline, True),
+                )
             else:
-                underline.highlight_start = start
-                underline.highlight_end = end
+                self.call_after_refresh(move_underline, False)
 
     async def _on_tab_clicked(self, event: Tab.Clicked) -> None:
         """Activate a tab that was clicked."""
@@ -671,7 +705,7 @@ class Tabs(Widget, can_focus=True):
         offset = event.offset + (0, -1)
         self.focus()
         for tab in self.query(Tab):
-            if offset in tab.region:
+            if offset in tab.region and not tab.disabled:
                 self._activate_tab(tab)
                 break
 
@@ -699,17 +733,21 @@ class Tabs(Widget, can_focus=True):
         self._move_tab(-1)
 
     def _move_tab(self, direction: int) -> None:
-        """Activate the next tab.
+        """Activate the next enabled tab in the given direction.
+
+        Tab selection wraps around. If no tab is currently active, the "next"
+        tab is set to be the first and the "previous" tab is the last one.
 
         Args:
             direction: +1 for the next tab, -1 for the previous.
         """
         active_tab = self.active_tab
-        if active_tab is None:
-            return
         tabs = self._potentially_active_tabs
         if not tabs:
             return
+        if not active_tab:
+            self.active = tabs[0 if direction == 1 else -1].id or ""
+            return
         tab_count = len(tabs)
         new_tab_index = (tabs.index(active_tab) + direction) % tab_count
         self.active = tabs[new_tab_index].id or ""
@@ -797,7 +835,7 @@ class Tabs(Widget, can_focus=True):
             next_tab = self._next_active
             self.active = next_tab.id or "" if next_tab else ""
         tab_to_hide.add_class("-hidden")
-        self.post_message(self.TabHidden(self, tab_to_hide))
+        self.post_message(self.TabHidden(self, tab_to_hide).set_sender(self))
         self.call_after_refresh(self._highlight_active)
         return tab_to_hide
 
@@ -820,7 +858,7 @@ class Tabs(Widget, can_focus=True):
             raise self.TabError(f"There is no tab with ID {tab_id!r} to show.")
 
         tab_to_show.remove_class("-hidden")
-        self.post_message(self.TabShown(self, tab_to_show))
+        self.post_message(self.TabShown(self, tab_to_show).set_sender(self))
         if not self.active:
             self._activate_tab(tab_to_show)
         self.call_after_refresh(self._highlight_active)
